#include <iostream>
#include <vector>
#include <mpi.h>
#include <cmath>

#include "Utils.hpp"

using namespace std;

#define ROOT_PROCESS 0

double computeSumOfSquares(const vector<double> &u) {
    double sumSquares = 0;
    for (double x : u) {
        sumSquares += x * x;
    }
    return sumSquares;
}

double computeVectorLength(const vector<double> u) {
    return sqrt(computeSumOfSquares(u));
}

void divideByVectorLength(vector<double> &u, double vectorLength) {
    for (int i = 0; i < u.size(); i++) {
        u[i] /= vectorLength;
    }
}

int main(int argc, char** argv) {
    MPI_Init(&argc, &argv);

    int myRank;
    MPI_Comm_rank(MPI_COMM_WORLD, &myRank);

    int worldSize;
    MPI_Comm_size(MPI_COMM_WORLD, &worldSize);

    if (myRank == ROOT_PROCESS) {
        vector<double> u = generateRandomVector(5000000);

        Stopwatch sw;
        sw.start();

        // TODO:

        vector<double> uNorm;   // TODO: fill here the normalized vector u

        sw.stop();
        cout << "MPI: " << sw.duration().count() << " ms, length " << computeVectorLength(uNorm) << endl;
    }
    else {
        // TODO:
    }

    MPI_Finalize();

    return 0;
}
